from qgis.core import QgsProcessing
from qgis.core import QgsProcessingAlgorithm
from qgis.core import QgsProcessingMultiStepFeedback
from qgis.core import QgsProcessingParameterVectorLayer
from qgis.core import QgsProcessingParameterBoolean
from qgis.core import QgsProcessingParameterFeatureSink
from qgis.core import QgsProcessingParameterNumber
import processing


class CentroidokBefoglaljbanVletlenPontokKpzseModelleszkz(QgsProcessingAlgorithm):

    def initAlgorithm(self, config=None):
        self.addParameter(QgsProcessingParameterVectorLayer('input', 'Bemeneti poligonos vektorréteg', types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterBoolean('ALLPARTS', 'A többelemű geometriák egyes részeire külön centroidot képezzek?', defaultValue=False))
        self.addParameter(QgsProcessingParameterFeatureSink('KimenetiRtegKonvexSokszg', 'Kimeneti réteg: konvex sokszög', type=QgsProcessing.TypeVectorAnyGeometry, createByDefault=True, defaultValue=None))
        self.addParameter(QgsProcessingParameterFeatureSink('KimenetiRtegVletlenPontok', 'Kimeneti réteg: véletlen pontok', type=QgsProcessing.TypeVectorPoint, createByDefault=True, defaultValue=None))
        self.addParameter(QgsProcessingParameterNumber('count', 'Véletlen pontok száma', type=QgsProcessingParameterNumber.Integer, defaultValue=1000))

    def processAlgorithm(self, parameters, context, model_feedback):
        # Use a multi-step feedback, so that individual child algorithm progress reports are adjusted for the
        # overall progress through the model
        feedback = QgsProcessingMultiStepFeedback(4, model_feedback)
        results = {}
        outputs = {}

        # Centrálisok
        alg_params = {
            'ALL_PARTS': parameters['ALLPARTS'],
            'INPUT': parameters['input'],
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        outputs['Centrlisok'] = processing.run('native:centroids', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.setCurrentStep(1)
        if feedback.isCanceled():
            return {}

        # Összevon
        alg_params = {
            'FIELD': None,
            'INPUT': outputs['Centrlisok']['OUTPUT'],
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        outputs['Sszevon'] = processing.run('native:dissolve', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.setCurrentStep(2)
        if feedback.isCanceled():
            return {}

        # Konvex körvonal
        alg_params = {
            'INPUT': outputs['Sszevon']['OUTPUT'],
            'OUTPUT': parameters['KimenetiRtegKonvexSokszg']
        }
        outputs['KonvexKrvonal'] = processing.run('native:convexhull', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        results['KimenetiRtegKonvexSokszg'] = outputs['KonvexKrvonal']['OUTPUT']

        feedback.setCurrentStep(3)
        if feedback.isCanceled():
            return {}

        # Véletlen pontok a felületen belül
        alg_params = {
            'INPUT': outputs['KonvexKrvonal']['OUTPUT'],
            'MIN_DISTANCE': None,
            'STRATEGY': 0,
            'VALUE': parameters['count'],
            'OUTPUT': parameters['KimenetiRtegVletlenPontok']
        }
        outputs['VletlenPontokAFelletenBell'] = processing.run('qgis:randompointsinsidepolygons', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        results['KimenetiRtegVletlenPontok'] = outputs['VletlenPontokAFelletenBell']['OUTPUT']
        return results

    def name(self):
        return 'Centroidok befoglalójában véletlen pontok képzése - modelleszköz'

    def displayName(self):
        return 'Centroidok befoglalójában véletlen pontok képzése - modelleszköz'

    def group(self):
        return 'Saját eszközcsoport'

    def groupId(self):
        return 'Saját eszközcsoport'

    def createInstance(self):
        return CentroidokBefoglaljbanVletlenPontokKpzseModelleszkz()
