from qgis.core import QgsProcessing
from qgis.core import QgsProcessingAlgorithm
from qgis.core import QgsProcessingMultiStepFeedback
from qgis.core import QgsProcessingParameterVectorLayer
from qgis.core import QgsProcessingParameterBoolean
from qgis.core import QgsProcessingParameterFeatureSink
from qgis.core import QgsProcessingParameterNumber
import processing


class centroidok_befoglalojaban_veletlen_pontok_kepzese_paros(QgsProcessingAlgorithm):

    def initAlgorithm(self, config=None):
        self.addParameter(QgsProcessingParameterVectorLayer('INPUT', 'Bemeneti poligonos vektorréteg', types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterBoolean('ALL_PARTS', 'A többelemű geometriák egyes részeire külön centroidot képezzek?', defaultValue=False))
        self.addParameter(QgsProcessingParameterNumber('COUNT', 'Véletlen pontok száma (páros legyen!)', type=QgsProcessingParameterNumber.Integer, defaultValue=1000))
        self.addParameter(QgsProcessingParameterFeatureSink('OUTPUT', 'Kimeneti réteg: véletlen pontok', type=QgsProcessing.TypeVectorPoint, createByDefault=True, defaultValue=None))
        self.addParameter(QgsProcessingParameterFeatureSink('CONVEX_HULL', 'Kimeneti réteg: konvex sokszög', type=QgsProcessing.TypeVectorAnyGeometry, createByDefault=True, defaultValue=None))

    def processAlgorithm(self, parameters, context, feedback):
        eredmenyszotar = {}
        kimenetek = {}

        megadott_darabszam = parameters['COUNT']
        if megadott_darabszam % 2 != 0:
            parameters['COUNT'] = megadott_darabszam - 1
            # feedback.pushWarning("A generálandó véletlen pontok száma csak páros lehet! A megadott értéket (" + str(megadott_darabszam) + ") csökkentettem " + str(parameters['COUNT']) + "-ra/-re.")
            feedback.pushInfo("FIGYELMEZTETÉS: A generálandó véletlen pontok száma csak páros lehet! A megadott értéket (" + str(megadott_darabszam) + ") csökkentettem " + str(parameters['COUNT']) + "-ra/-re.")

        # Centrálisok
        alg_params = {
            'ALL_PARTS': parameters['ALL_PARTS'],
            'INPUT': parameters['INPUT'],
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        kimenetek['Centrlisok'] = processing.run('native:centroids', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.pushInfo("Centroidok képzése kész.")
        if feedback.isCanceled():
            return {}

        # Összevon
        alg_params = {
            'FIELD': None,
            'INPUT': kimenetek['Centrlisok']['OUTPUT'],
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        kimenetek['Sszevon'] = processing.run('native:dissolve', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.pushInfo("Összevonás kész.")
        if feedback.isCanceled():
            return {}

        # Konvex körvonal
        alg_params = {
            'INPUT': kimenetek['Sszevon']['OUTPUT'],
            'OUTPUT': parameters['CONVEX_HULL']
        }
        kimenetek['KonvexKrvonal'] = processing.run('native:convexhull', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.pushInfo("Legkisebb befoglaló konvex sokszög képzése kész.")
        if feedback.isCanceled():
            return {}

        # Véletlen pontok a felületen belül
        alg_params = {
            'INPUT': kimenetek['KonvexKrvonal']['OUTPUT'],
            'MIN_DISTANCE': None,
            'STRATEGY': 0,
            'VALUE': parameters['COUNT'],
            'OUTPUT': parameters['OUTPUT']
        }
        kimenetek['VletlenPontokAFelletenBell'] = processing.run('qgis:randompointsinsidepolygons', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        feedback.pushInfo("Véletlen pontok generálása kész.")
        eredmenyszotar['OUTPUT'] = kimenetek['VletlenPontokAFelletenBell']['OUTPUT']
        eredmenyszotar['CONVEX_HULL'] = kimenetek['KonvexKrvonal']['OUTPUT']
        return eredmenyszotar

    def name(self):
        return 'centroid_befoglalo_random_paros'

    def displayName(self):
        return 'Centroidok befoglalójában véletlen pontok képzése - páros'

    def group(self):
        return 'Saját eszközcsoport'

    def groupId(self):
        return 'sajat_eszkozcsoport'

    def createInstance(self):
        return centroidok_befoglalojaban_veletlen_pontok_kepzese_paros()
