from qgis.core import QgsProcessing
from qgis.core import QgsProcessingAlgorithm
from qgis.core import QgsProcessingMultiStepFeedback
from qgis.core import QgsProcessingParameterVectorLayer
from qgis.core import QgsProcessingParameterFile
from qgis.core import QgsCoordinateReferenceSystem
import processing


class kml_es_gpx_mentese(QgsProcessingAlgorithm):

    def initAlgorithm(self, config=None):
        self.addParameter(QgsProcessingParameterVectorLayer('INPUT', 'bementi vektorréteg', types=[QgsProcessing.TypeVector], defaultValue=None))
        self.addParameter(QgsProcessingParameterFile('FOLDER', 'kimeneti mappa', optional=True, behavior=QgsProcessingParameterFile.Folder, fileFilter='Minden fájl (*.*)', defaultValue=None))

    def processAlgorithm(self, parameterek, context, feedback):
        eredmenyszotar = {}
        kimenetek = {}

        # Réteg más vetületbe
        alg_params = {
            'INPUT': parameterek['INPUT'],
            'TARGET_CRS': QgsCoordinateReferenceSystem('EPSG:4326'),
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        kimenetek['atvetites'] = processing.run('native:reprojectlayer', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        if feedback.isCanceled():
            return {}

        # Mezők törlése
        alg_params = {
            'COLUMN': 'valami',
            'INPUT': kimenetek['atvetites']['OUTPUT'],
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        kimenetek['mezotorles'] = processing.run('qgis:deletecolumn', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        if feedback.isCanceled():
            return {}

        # Kiválasztott elemek átvétele
        alg_params = {
            'INPUT': kimenetek['mezotorles']['OUTPUT'],
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        kimenetek['mentes_kml'] = processing.run('native:saveselectedfeatures', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        if feedback.isCanceled():
            return {}

        # Kiválasztott elemek átvétele
        alg_params = {
            'INPUT': kimenetek['mezotorles']['OUTPUT'],
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        kimenetek['mentes_gpx'] = processing.run('native:saveselectedfeatures', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        return eredmenyszotar

    def name(self):
        return 'kml_es_gpx_mentese'

    def displayName(self):
        return 'Kml és gpx mentése'

    def group(self):
        return 'Saját eszközcsoport'

    def groupId(self):
        return 'sajat_eszkozcsoport'

    def createInstance(self):
        return kml_es_gpx_mentese()
