from qgis.core import QgsProcessing
from qgis.core import QgsProcessingAlgorithm
from qgis.core import QgsProcessingMultiStepFeedback
from qgis.core import QgsProcessingParameterVectorLayer
from qgis.core import QgsProcessingParameterFile
from qgis.core import QgsCoordinateReferenceSystem
from qgis.core import QgsProcessingFeatureSourceDefinition
from qgis.core import QgsVectorLayer
from os.path import dirname
import processing


class kml_es_gpx_mentese(QgsProcessingAlgorithm):

	def initAlgorithm(self, config=None):
		self.addParameter(QgsProcessingParameterVectorLayer('INPUT', 'bementi vektorréteg', types=[QgsProcessing.TypeVector], defaultValue=None))
		self.addParameter(QgsProcessingParameterFile('FOLDER', 'kimeneti mappa', optional=True, behavior=QgsProcessingParameterFile.Folder, fileFilter='Minden fájl (*.*)', defaultValue=" "))

	def processAlgorithm(self, parameterek, context, feedback):
		eredmenyszotar = {}
		kimenetek = {}

		bemeneti_vektorreteg = self.parameterAsVectorLayer(parameterek, 'INPUT', context) # bemeneti parameter (szöveg) elkapása rétegként
		nincs_kijeloles = len(bemeneti_vektorreteg.selectedFeatures()) == 0 # 0 elem van-e kijelölve
		if nincs_kijeloles: # ha nincs kijelölés
			bemeneti_vektorreteg.selectAll() # mindet kijelöli
			feedback.pushInfo("Ideiglenesen kijelöltem minden elemet.")

		# Réteg más vetületbe
		alg_params = {
			'INPUT': QgsProcessingFeatureSourceDefinition(
				source = parameterek['INPUT'],
				selectedFeaturesOnly = True),
			'TARGET_CRS': QgsCoordinateReferenceSystem('EPSG:4326'),
			'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
		}

		szukseges_atvetiteni = bemeneti_vektorreteg.crs() != QgsCoordinateReferenceSystem('EPSG:4326') # szükséges-e az átvetítés
		if not szukseges_atvetiteni: # ha már eleve WGS-84-ben van
			hasznalando_reteg_neve = parameterek['INPUT']
			feedback.pushInfo("Átvetítést kihagytam.")
		else:
			kimenetek['atvetites'] = processing.run('native:reprojectlayer', alg_params, context=context, feedback=feedback, is_child_algorithm=True) # átvetítés
			hasznalando_reteg_neve = kimenetek['atvetites']['OUTPUT']
			feedback.pushInfo("Átvetítést elvégeztem.")

		if feedback.isCanceled():
			return {}

		atvetitett_vektorreteg = context.getMapLayer(hasznalando_reteg_neve) # szövegből réteggé alakítás
		osszes_mezo_neve = []
		for mezo in atvetitett_vektorreteg.fields(): # végiglépked a mezőkön
			osszes_mezo_neve.append(mezo.name()) # a mező nevét a listához fűzi
		feedback.pushInfo("Mezőneveket kigyűjtöttem.")

		if not szukseges_atvetiteni: # ha már eleve WGS-84-ben van
			atvetitett_vektorreteg = QgsProcessingFeatureSourceDefinition(source = parameterek['INPUT'], selectedFeaturesOnly = True) # csak a kijelölt elemekkel dolgozik tovább

		# Mezők törlése
		alg_params = {
			'COLUMN': osszes_mezo_neve,
			'INPUT': atvetitett_vektorreteg,
			'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
		}
		kimenetek['mezotorles'] = processing.run('qgis:deletecolumn', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
		feedback.pushInfo("Mezőket töröltem.")

		if feedback.isCanceled():
			return {}

		megadott_mappat = not (parameterek['FOLDER'] == " " or parameterek['FOLDER'] == "") # megadott-e a felhasználó mappát
		if megadott_mappat: # ha megadott mappát
			kimeneti_mappa = parameterek['FOLDER'] # az lesz a mentés helye
			feedback.pushInfo("A megadott mappát használom a mentéshez.")
		else: # ha nem adott meg mappát
			bemeneti_vektor_forrasfajlja = bemeneti_vektorreteg.source() # a bemeneti vektor elérési útja és fájlneve kiterjesztéssel
			kimeneti_mappa = dirname(bemeneti_vektor_forrasfajlja) # csak az elérési út
			feedback.pushInfo("A bemeneti vektorréteget tartalmazó mappát használom a mentéshez.")

		mezoktol_megfosztott_vektorreteg = context.getMapLayer(kimenetek['mezotorles']['OUTPUT']) # szövegből réteggé alakítás
		mezoktol_megfosztott_vektorreteg.selectAll() # mindet kijelöli

		# Kiválasztott elemek átvétele
		alg_params = {
			'INPUT': kimenetek['mezotorles']['OUTPUT'],
			'OUTPUT': kimeneti_mappa + "/" + bemeneti_vektorreteg.name() + ".kml"
		}
		kimenetek['mentes_kml'] = processing.run('native:saveselectedfeatures', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
		feedback.pushInfo("KML-fájlt elmentettem.")

		if feedback.isCanceled():
			return {}

		# Kiválasztott elemek átvétele
		alg_params = {
			'INPUT': kimenetek['mezotorles']['OUTPUT'],
			'OUTPUT': kimeneti_mappa + "/" + bemeneti_vektorreteg.name() + ".gpx"
		}
		kimenetek['mentes_gpx'] = processing.run('native:saveselectedfeatures', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
		feedback.pushInfo("GPX-fájlt elmentettem.")
		
		if nincs_kijeloles: # ha eredetileg nem volt kijelölés
			bemeneti_vektorreteg.removeSelection()  # megszünteti a kijelölést
			feedback.pushInfo("Az ideiglenes kijelölést megszüntettem.")

		return eredmenyszotar

	def name(self):
		return 'kml_es_gpx_mentese'

	def displayName(self):
		return 'Kml és gpx mentése'

	def group(self):
		return 'Saját eszközcsoport'

	def groupId(self):
		return 'sajat_eszkozcsoport'

	def createInstance(self):
		return kml_es_gpx_mentese()
